@@ -39,10 +39,6 @@
 #define LOG_FMT_RPATH			"FORMAT: R - Canonical Path\n"
 #define LOG_FMT_CMDLINE			"FORMAT: C - Command Line\n"
 
-#define PROC_DIR			"/proc"
-#define PROC_SELF_FD			PROC_DIR "/self/fd"
-#define PROC_SELF_CMDLINE		PROC_DIR "/self/cmdline"
-
 char sandbox_lib[SB_PATH_MAX];
 
 typedef struct {
@@ -64,7 +60,7 @@ typedef struct {
 
 static sbcontext_t sbcontext;
 static char *cached_env_vars[MAX_DYN_PREFIXES];
-volatile bool sandbox_on = true;
+bool sandbox_on = true;
 static bool sb_init = false;
 
 static char *resolve_path(const char *, int);
@@ -116,6 +112,29 @@ void libsb_init(void)
 	errno = old_errno;
 }
 
+static const char *sb_get_fd_dir(void)
+{
+#if defined(SANDBOX_PROC_SELF_FD)
+	return "/proc/self/fd";
+#elif defined(SANDBOX_DEV_FD)
+	return "/dev/fd";
+#else
+# error "how do i access a proc's fd/ tree ?"
+#endif
+}
+
+static const char *sb_get_cmdline(pid_t pid)
+{
+#if !defined(SANDBOX_PROC_1_CMDLINE)
+# error "how do i access a proc's cmdline ?"
+#endif
+	static char path[256];
+	if (!pid)
+		pid = getpid();
+	sprintf(path, "/proc/%i/cmdline", pid);
+	return path;
+}
+
 int canonicalize(const char *path, char *resolved_path)
 {
 	int old_errno = errno;
@@ -194,7 +213,13 @@ static char *resolve_path(const char *path, int follow_link)
 		 * etc.  If that fails (might not exist), we try to get the
 		 * realpath of the parent directory, as that should hopefully
 		 * exist.  If all else fails, just go with canonicalize */
-		if (NULL == realpath(path, filtered_path)) {
+		char *ret;
+		if (trace_pid)
+			ret = erealpath(path, filtered_path);
+		else
+			ret = realpath(path, filtered_path);
+
+		if (!ret) {
 			char tmp_str1[SB_PATH_MAX];
 			snprintf(tmp_str1, SB_PATH_MAX, "%s", path);
 
@@ -202,7 +227,11 @@ static char *resolve_path(const char *path, int follow_link)
 
 			/* If not, then check if we can resolve the
 			 * parent directory */
-			if (NULL == realpath(dname, filtered_path)) {
+			if (trace_pid)
+				ret = erealpath(dname, filtered_path);
+			else
+				ret = realpath(dname, filtered_path);
+			if (!ret) {
 				/* Fall back to canonicalize */
 				if (-1 == canonicalize(path, filtered_path)) {
 					free(filtered_path);
@@ -243,6 +272,17 @@ char *egetcwd(char *buf, size_t size)
 	struct stat st;
 	char *tmpbuf, *oldbuf = buf;
 
+	/* If tracing a child, our cwd may not be the same as the child's */
+	if (trace_pid) {
+		char proc[20];
+		sprintf(proc, "/proc/%i/cwd", trace_pid);
+		ssize_t ret = readlink(proc, buf, size);
+		if (ret == -1)
+			return NULL;
+		buf[ret] = '\0';
+		return buf;
+	}
+
 	/* Need to disable sandbox, as on non-linux libc's, opendir() is
 	 * used by some getcwd() implementations and resolves to the sandbox
 	 * opendir() wrapper, causing infinit recursion and finially crashes.
@@ -354,7 +394,7 @@ static bool write_logfile(const char *logfile, const char *func, const char *pat
 	    (0 == S_ISREG(log_stat.st_mode))) {
 		SB_EERROR("SECURITY BREACH", "  '%s' %s\n", logfile,
 			"already exists and is not a regular file!");
-		abort();
+		sb_abort();
 	}
 
 	logfd = sb_open(logfile,
@@ -394,7 +434,8 @@ static bool write_logfile(const char *logfile, const char *func, const char *pat
 	_SB_WRITE_STR(rpath);
 
 	_SB_WRITE_STR("\nC: ");
-	int cmdlinefd = sb_open(PROC_SELF_CMDLINE, O_RDONLY, 0);
+	const char *cmdline = sb_get_cmdline(trace_pid);
+	int cmdlinefd = sb_open(cmdline, O_RDONLY, 0);
 	if (cmdlinefd != -1) {
 		size_t pagesz = getpagesize();
 		char *buf = xmalloc(pagesz);
@@ -413,8 +454,10 @@ static bool write_logfile(const char *logfile, const char *func, const char *pat
 		}
 		sb_close(cmdlinefd);
 		free(buf);
-	} else
-		_SB_WRITE_STR("<unable to read " PROC_SELF_CMDLINE ">");
+	} else {
+		_SB_WRITE_STR("unable to read ");
+		_SB_WRITE_STR(cmdline);
+	}
 	_SB_WRITE_STR("\n");
 
 	return true;
@@ -634,36 +677,41 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func,
 	}
 
 	if (sb_nr == SB_NR_ACCESS_WR   ||
-	    sb_nr == SB_NR_OPEN_WR     ||
+	    sb_nr == SB_NR_CHMOD       ||
+	    sb_nr == SB_NR_CHOWN       ||
 	    sb_nr == SB_NR_CREAT       ||
 	    sb_nr == SB_NR_CREAT64     ||
+	    sb_nr == SB_NR_FCHMODAT    ||
+	    sb_nr == SB_NR_FCHOWNAT    ||
+	  /*sb_nr == SB_NR_FTRUNCATE   ||
+	    sb_nr == SB_NR_FTRUNCATE64 ||*/
+	    sb_nr == SB_NR_FUTIMESAT   ||
+	    sb_nr == SB_NR_LCHOWN      ||
+	    sb_nr == SB_NR_LINK        ||
+	    sb_nr == SB_NR_LINKAT      ||
+	    sb_nr == SB_NR_LUTIMES     ||
 	    sb_nr == SB_NR_MKDIR       ||
+	    sb_nr == SB_NR_MKDIRAT     ||
+	    sb_nr == SB_NR_MKFIFO      ||
+	    sb_nr == SB_NR_MKFIFOAT    ||
 	    sb_nr == SB_NR_MKNOD       ||
 	    sb_nr == SB_NR_MKNODAT     ||
-	    sb_nr == SB_NR__XMKNOD     ||
-	    sb_nr == SB_NR___XMKNOD    ||
-	    sb_nr == SB_NR___XMKNODAT  ||
-	    sb_nr == SB_NR_MKFIFO      ||
-	    sb_nr == SB_NR_LINK        ||
-	    sb_nr == SB_NR_SYMLINK     ||
+	    sb_nr == SB_NR_OPEN_WR     ||
 	    sb_nr == SB_NR_RENAME      ||
-	    sb_nr == SB_NR_LUTIMES     ||
-	    sb_nr == SB_NR_UTIMENSAT   ||
-	    sb_nr == SB_NR_UTIME       ||
-	    sb_nr == SB_NR_UTIMES      ||
-	    sb_nr == SB_NR_FUTIMESAT   ||
+	    sb_nr == SB_NR_RENAMEAT    ||
+	    sb_nr == SB_NR_RMDIR       ||
+	    sb_nr == SB_NR_SYMLINK     ||
+	    sb_nr == SB_NR_SYMLINKAT   ||
+	    sb_nr == SB_NR_TRUNCATE    ||
+	    sb_nr == SB_NR_TRUNCATE64  ||
 	    sb_nr == SB_NR_UNLINK      ||
 	    sb_nr == SB_NR_UNLINKAT    ||
-	    sb_nr == SB_NR_RMDIR       ||
-	    sb_nr == SB_NR_CHOWN       ||
-	    sb_nr == SB_NR_FCHOWNAT    ||
-	    sb_nr == SB_NR_LCHOWN      ||
-	    sb_nr == SB_NR_CHMOD       ||
-	    sb_nr == SB_NR_FCHMODAT    ||
-	    sb_nr == SB_NR_TRUNCATE    ||
-	  /*sb_nr == SB_NR_FTRUNCATE   ||*/
-	    sb_nr == SB_NR_TRUNCATE64/*||
-	    sb_nr == SB_NR_FTRUNCATE64*/)
+	    sb_nr == SB_NR_UTIME       ||
+	    sb_nr == SB_NR_UTIMENSAT   ||
+	    sb_nr == SB_NR_UTIMES      ||
+	    sb_nr == SB_NR__XMKNOD     ||
+	    sb_nr == SB_NR___XMKNOD    ||
+	    sb_nr == SB_NR___XMKNODAT)
 	{
 
 		retval = check_prefixes(sbcontext->write_denied_prefixes,
@@ -726,8 +774,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func,
 		 * will differ ...
 		 */
 		char proc_self_fd[SB_PATH_MAX];
-		if (!strncmp(resolv_path, PROC_DIR, strlen(PROC_DIR)) &&
-		    NULL != realpath(PROC_SELF_FD, proc_self_fd) &&
+		if (realpath(sb_get_fd_dir(), proc_self_fd) &&
 		    !strncmp(resolv_path, proc_self_fd, strlen(proc_self_fd)))
 		{
 			result = 1;
@@ -915,7 +962,7 @@ bool before_syscall(int dirfd, int sb_nr, const char *func, const char *file, in
 	 */
 	if (dirfd != AT_FDCWD && file[0] != '/') {
 		size_t at_len = sizeof(at_file_buf) - 1 - 1 - strlen(file);
-		sprintf(at_file_buf, "/proc/%i/fd/%i", getpid(), dirfd);
+		sprintf(at_file_buf, "/proc/%i/fd/%i", trace_pid ? : getpid(), dirfd);
 		ssize_t ret = readlink(at_file_buf, at_file_buf, at_len);
 		if (ret == -1) {
 			/* see comments at end of check_syscall() */
@@ -923,6 +970,12 @@ bool before_syscall(int dirfd, int sb_nr, const char *func, const char *file, in
 				restore_errno();
 				return true;
 			}
+			if (is_env_on(ENV_SANDBOX_DEBUG))
+				SB_EINFO("AT_FD LOOKUP", "  fail: %s: %s\n",
+					at_file_buf, strerror(errno));
+			/* If the fd isn't found, some guys (glibc) expect errno */
+			if (errno == ENOENT)
+				errno = EBADF;
 			return false;
 		}
 		at_file_buf[ret] = '/';
