@@ -14,8 +14,7 @@
 	char *file = s; \
 	\
 	s = argv[i++]; \
-	mode_t mode; \
-	sscanf(s, "%i", &mode); \
+	mode_t mode = sscanf_mode_t(s); \
 	\
 	s = argv[i++]; \
 	int flags = at_get_flags(s);
